@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -955,14 +955,6 @@ gx_default_draw_thin_line(gx_device * dev,
     }
 }
 
-/* Stub out the obsolete procedure. */
-int
-gx_default_draw_line(gx_device * dev,
-                     int x0, int y0, int x1, int y1, gx_color_index color)
-{
-    return -1;
-}
-
 /* ---------------- Image drawing ---------------- */
 
 /* GC structures for image enumerator */
@@ -980,51 +972,6 @@ static RELOC_PTRS_WITH(image_enum_common_reloc_ptrs, gx_image_enum_common_t *ept
 }
 RELOC_PTRS_END
 
-/*
- * gx_default_begin_image is only invoked for ImageType 1 images.  However,
- * the argument types are different, and if the device provides a
- * begin_typed_image procedure, we should use it.  See gxdevice.h.
- */
-static int
-gx_no_begin_image(gx_device * dev,
-                  const gs_gstate * pgs, const gs_image_t * pim,
-                  gs_image_format_t format, const gs_int_rect * prect,
-              const gx_drawing_color * pdcolor, const gx_clip_path * pcpath,
-                  gs_memory_t * memory, gx_image_enum_common_t ** pinfo)
-{
-    return -1;
-}
-int
-gx_default_begin_image(gx_device * dev,
-                       const gs_gstate * pgs, const gs_image_t * pim,
-                       gs_image_format_t format, const gs_int_rect * prect,
-              const gx_drawing_color * pdcolor, const gx_clip_path * pcpath,
-                       gs_memory_t * memory, gx_image_enum_common_t ** pinfo)
-{
-    /*
-     * Hand off to begin_typed_image, being careful to avoid a
-     * possible recursion loop.
-     */
-    dev_proc_begin_image((*save_begin_image)) = dev_proc(dev, begin_image);
-    gs_image_t image;
-    const gs_image_t *ptim;
-    int code;
-
-    set_dev_proc(dev, begin_image, gx_no_begin_image);
-    if (pim->format == format)
-        ptim = pim;
-    else {
-        image = *pim;
-        image.format = format;
-        ptim = &image;
-    }
-    code = (*dev_proc(dev, begin_typed_image))
-        (dev, pgs, NULL, (const gs_image_common_t *)ptim, prect, pdcolor,
-         pcpath, memory, pinfo);
-    set_dev_proc(dev, begin_image, save_begin_image);
-    return code;
-}
-
 int
 gx_default_begin_typed_image(gx_device * dev,
                         const gs_gstate * pgs, const gs_matrix * pmat,
@@ -1032,44 +979,10 @@ gx_default_begin_typed_image(gx_device * dev,
               const gx_drawing_color * pdcolor, const gx_clip_path * pcpath,
                       gs_memory_t * memory, gx_image_enum_common_t ** pinfo)
 {
-    /* If this is an ImageType 1 image using the gs_gstate's CTM,
-         * defer to begin_image.
-         */
-    if (pic->type->begin_typed_image == gx_begin_image1) {
-        const gs_image_t *pim = (const gs_image_t *)pic;
-
-        if (pmat == 0 ||
-            (pgs != 0 && !gs_matrix_compare(pmat, &ctm_only(pgs)))
-            ) {
-            int code = (*dev_proc(dev, begin_image))
-            (dev, pgs, pim, pim->format, prect, pdcolor,
-             pcpath, memory, pinfo);
-
-            if (code >= 0)
-                return code;
-        }
-    }
     return (*pic->type->begin_typed_image)
         (dev, pgs, pmat, pic, prect, pdcolor, pcpath, memory, pinfo);
 }
 
-/* Backward compatibility for obsolete driver procedures. */
-
-int
-gx_default_image_data(gx_device *dev, gx_image_enum_common_t * info,
-                      const byte ** plane_data,
-                      int data_x, uint raster, int height)
-{
-    return gx_image_data(info, plane_data, data_x, raster, height);
-}
-
-int
-gx_default_end_image(gx_device *dev, gx_image_enum_common_t * info,
-                     bool draw_last)
-{
-    return gx_image_end(info, draw_last);
-}
-
 int
 gx_default_fillpage(gx_device *dev, gs_gstate * pgs, gx_device_color *pdevc)
 {
