@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2019 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -23,7 +23,6 @@
 #include "gxdcolor.h"
 #include "gxdevice.h"
 #include "gxdevmem.h"
-#include "gxdevrop.h"
 #include "gdevmem.h"
 #include "gdevmrop.h"
 
@@ -47,12 +46,13 @@
 /* ---------------- RasterOp with 8-bit gray / 24-bit RGB ---------------- */
 
 int
-mem_gray8_rgb24_strip_copy_rop(gx_device * dev,
+mem_gray8_rgb24_strip_copy_rop2(gx_device * dev,
              const byte * sdata, int sourcex, uint sraster, gx_bitmap_id id,
                                const gx_color_index * scolors,
            const gx_strip_bitmap * textures, const gx_color_index * tcolors,
                                int x, int y, int width, int height,
-                       int phase_x, int phase_y, gs_logical_operation_t dirty_lop)
+                       int phase_x, int phase_y,
+                       gs_logical_operation_t dirty_lop, uint planar_height)
 {
     gx_device_memory *mdev = (gx_device_memory *) dev;
     gs_logical_operation_t lop = lop_sanitize(dirty_lop);
@@ -68,12 +68,6 @@ mem_gray8_rgb24_strip_copy_rop(gx_device * dev,
     gx_color_index local_scolors[2];
     gx_color_index local_tcolors[2];
 
-#if !defined(USE_RUN_ROP) || defined(COMPARE_AND_CONTRAST)
-    gx_color_index strans =
-        (lop & lop_S_transparent ? all_ones : gx_no_color_index);
-    gx_color_index ttrans =
-        (lop & lop_T_transparent ? all_ones : gx_no_color_index);
-#endif
 #ifdef USE_RUN_ROP
     rop_run_op ropper;
 #ifdef COMPARE_AND_CONTRAST
@@ -83,8 +77,10 @@ mem_gray8_rgb24_strip_copy_rop(gx_device * dev,
 #endif
 #endif
 
+    /* assert(planar_height == 0); */
+
     /* Check for constant source. */
-    if (!rop3_uses_S(lop) && (lop & lop_S_transparent) == 0) {
+    if (!rop3_uses_S(lop)) {
         const_source = 0;       /* arbitrary */
     } else if (scolors != 0 && scolors[0] == scolors[1]) {
         /* Constant source */
@@ -92,14 +88,12 @@ mem_gray8_rgb24_strip_copy_rop(gx_device * dev,
         if (const_source == gx_device_black(dev))
             lop = lop_know_S_0(lop);
         else if (const_source == gx_device_white(dev)) {
-            if (lop & lop_S_transparent)
-                return 0;
             lop = lop_know_S_1(lop);
         }
     }
 
     /* Check for constant texture. */
-    if (!rop3_uses_T(lop) && (lop & lop_T_transparent) == 0)
+    if (!rop3_uses_T(lop))
         const_texture = 0;      /* arbitrary */
     else if (tcolors != 0 && tcolors[0] == tcolors[1]) {
         /* Constant texture */
@@ -107,8 +101,6 @@ mem_gray8_rgb24_strip_copy_rop(gx_device * dev,
         if (const_texture == gx_device_black(dev))
             lop = lop_know_T_0(lop);
         else if (const_texture == gx_device_white(dev)) {
-            if (lop & lop_T_transparent)
-                return 0;
             lop = lop_know_T_1(lop);
         }
     }
@@ -142,22 +134,18 @@ bw:         if (bw_pixel == 0x00)
         case rop3_D:
             break;
         case rop3_S:
-            if (lop & lop_S_transparent)
-                goto df;
             break;
         case rop3_T:
-            if (lop & lop_T_transparent)
-                goto df;
             break;
         default:
-df:         return mem_default_strip_copy_rop(dev,
-                                              sdata, sourcex, sraster, id,
-                                              scolors, textures, tcolors,
-                                              x, y, width, height,
-                                              phase_x, phase_y, lop);
+df:         return mem_default_strip_copy_rop2(dev,
+                                               sdata, sourcex, sraster, id,
+                                               scolors, textures, tcolors,
+                                               x, y, width, height,
+                                               phase_x, phase_y, lop, 0);
         }
         /* Put the updated rop back into the lop */
-        lop = rop | (lop & (lop_S_transparent | lop_T_transparent));
+        lop = rop;
     }
 
     /* Adjust coordinates to be in bounds. */
@@ -209,10 +197,6 @@ df:         return mem_default_strip_copy_rop(dev,
   (dbit(base, i) ? (byte)colors[1] : (byte)colors[0])
 #ifdef COMPARE_AND_CONTRAST
 #define rop_body_8(s_pixel, t_pixel)\
-  if ( (s_pixel) == strans ||   /* So = 0, s_tr = 1 */\
-       (t_pixel) == ttrans      /* Po = 0, p_tr = 1 */\
-     )\
-    continue;\
   *dptr = (*rop_proc_table[rop])(*dptr, s_pixel, t_pixel)
 #endif
 /* 24-bit */
@@ -226,10 +210,6 @@ df:         return mem_default_strip_copy_rop(dev,
   (dbit(base, i) ? colors[1] : colors[0])
 #ifdef COMPARE_AND_CONTRAST
 #define rop_body_24(s_pixel, t_pixel)\
-  if ( (s_pixel) == strans ||   /* So = 0, s_tr = 1 */\
-       (t_pixel) == ttrans      /* Po = 0, p_tr = 1 */\
-     )\
-    continue;\
   { gx_color_index d_pixel = get24(dptr);\
     d_pixel = (*rop_proc_table[rop])(d_pixel, s_pixel, t_pixel);\
     put24(dptr, d_pixel);\
@@ -374,6 +354,7 @@ df:         return mem_default_strip_copy_rop(dev,
             int ty = y + phase_y;
 
             rop_set_s_constant(&ropper, const_source);
+            rop_set_t_colors(&ropper, tcolors);
             if (rop_get_run_op(&ropper, lop, depth, rop_s_constant | rop_t_1bit)) {
                 for (; line_count-- > 0; drow += draster, ++ty) {   /* Loop over copies of the tile. */
                     int dx = x, w = width, nw;
@@ -625,10 +606,6 @@ df:         return mem_default_strip_copy_rop(dev,
 #define cbit8(base, i, colors)\
   (dbit(base, i) ? (byte)colors[1] : (byte)colors[0])
 #define rop_body_8(s_pixel, t_pixel)\
-  if ( (s_pixel) == strans ||   /* So = 0, s_tr = 1 */\
-       (t_pixel) == ttrans      /* Po = 0, p_tr = 1 */\
-     )\
-    continue;\
   *dptr = (*rop_proc_table[rop])(*dptr, s_pixel, t_pixel)
 /* 24-bit */
 #define get24(ptr)\
@@ -640,10 +617,6 @@ df:         return mem_default_strip_copy_rop(dev,
 #define cbit24(base, i, colors)\
   (dbit(base, i) ? colors[1] : colors[0])
 #define rop_body_24(s_pixel, t_pixel)\
-  if ( (s_pixel) == strans ||   /* So = 0, s_tr = 1 */\
-       (t_pixel) == ttrans      /* Po = 0, p_tr = 1 */\
-     )\
-    continue;\
   { gx_color_index d_pixel = get24(dptr);\
     d_pixel = (*rop_proc_table[rop])(d_pixel, s_pixel, t_pixel);\
     put24(dptr, d_pixel);\
