@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -106,8 +106,8 @@ iclass_proc(gs_image_class_2_fracs);
 static irender_proc(image_render_frac);
 static irender_proc(image_render_icc16); /* icc 16bit case */
 
-irender_proc_t
-gs_image_class_2_fracs(gx_image_enum * penum)
+int
+gs_image_class_2_fracs(gx_image_enum * penum, irender_proc_t *render_fn)
 {
     bool std_cmap_procs;
 
@@ -134,7 +134,8 @@ gs_image_class_2_fracs(gx_image_enum * penum)
                a simple color space that just is scaled to the device bit
                depth when remapped. No CM needed */
             if_debug0m('b', penum->memory, "[b]render=frac\n");
-            return &image_render_frac;
+            *render_fn =  &image_render_frac;
+            return 0;
         } else {
             /* Set up the link now */
             const gs_color_space *pcs;
@@ -171,7 +172,7 @@ gs_image_class_2_fracs(gx_image_enum * penum)
             }
             penum->icc_setup.is_lab = pcs->cmm_icc_profile_data->islab;
             penum->icc_setup.must_halftone = gx_device_must_halftone(penum->dev);
-            penum->icc_setup.has_transfer = 
+            penum->icc_setup.has_transfer =
                 gx_has_transfer(penum->pgs, num_des_comps);
             if (penum->icc_setup.is_lab) penum->icc_setup.need_decode = false;
             if (penum->icc_link == NULL) {
@@ -181,7 +182,8 @@ gs_image_class_2_fracs(gx_image_enum * penum)
             /* Use the direct unpacking proc */
             penum->unpack = sample_unpackicc_16;
             if_debug0m('b', penum->memory, "[b]render=icc16\n");
-            return &image_render_icc16;
+            *render_fn = &image_render_icc16;
+            return 0;
         }
     }
     return 0;
@@ -262,7 +264,7 @@ image_render_frac(gx_image_enum * penum, const byte * buffer, int data_x,
     int code = 0, mcode = 0;
     int i;
     bool is_devn = false;
-    bool is_sep = (gs_color_space_get_index(penum->pcs) == 
+    bool is_sep = (gs_color_space_get_index(penum->pcs) ==
                     gs_color_space_index_Separation);
 
     if (h == 0)
@@ -410,14 +412,14 @@ image_render_frac(gx_image_enum * penum, const byte * buffer, int data_x,
 f:
         if (sizeof(pdevc_next->colors.binary.color[0]) <= sizeof(ulong))
             if_debug7m('B', penum->memory,
-                       "[B]0x%x,0x%x,0x%x,0x%x -> 0x%lx,0x%lx,0x%lx\n",
+                       "[B]0x%x,0x%x,0x%x,0x%x -> 0x%lx,0x%lx," PRI_INTPTR "\n",
                        next.v[0], next.v[1], next.v[2], next.v[3],
                        (ulong)pdevc_next->colors.binary.color[0],
                        (ulong)pdevc_next->colors.binary.color[1],
-                       (ulong)pdevc_next->type);
+                       (intptr_t)pdevc_next->type);
         else
             if_debug9m('B', penum->memory,
-                       "[B]0x%x,0x%x,0x%x,0x%x -> 0x%08lx%08lx,0x%08lx%08lx,0x%lx\n",
+                       "[B]0x%x,0x%x,0x%x,0x%x -> 0x%08lx%08lx,0x%08lx%08lx," PRI_INTPTR "\n",
                        next.v[0], next.v[1], next.v[2], next.v[3],
                        (ulong)(pdevc_next->colors.binary.color[0] >>
                                8 * (sizeof(pdevc_next->colors.binary.color[0]) - sizeof(ulong))),
@@ -425,7 +427,7 @@ f:
                        (ulong)(pdevc_next->colors.binary.color[1] >>
                                8 * (sizeof(pdevc_next->colors.binary.color[1]) - sizeof(ulong))),
                        (ulong)pdevc_next->colors.binary.color[1],
-                       (ulong)pdevc_next->type);
+                       (intptr_t)pdevc_next->type);
 /* NB: sizeof gx_color_index is 4 or 8 bytes! */
 
         /* Even though the supplied colors don't match, */
@@ -605,6 +607,7 @@ image_render_icc16(gx_image_enum * penum, const byte * buffer, int data_x,
     gx_cmapper_t data;
     gx_cmapper_fn *mapper;
     gx_color_value *conc = &data.conc[0];
+    int first = 1;
 
     if (h == 0)
         return 0;
@@ -663,19 +666,23 @@ image_render_icc16(gx_image_enum * penum, const byte * buffer, int data_x,
                                         (const unsigned short*) (psrc_decode+w),
                                          get_cie_range(penum->pcs));
                 }
-                (penum->icc_link->procs.map_buffer)(dev, penum->icc_link, 
+                code = (penum->icc_link->procs.map_buffer)(dev, penum->icc_link,
                                                     &input_buff_desc,
-                                                    &output_buff_desc, 
+                                                    &output_buff_desc,
                                                     (void*) psrc_decode,
                                                     (void*) psrc_cm);
                 gs_free_object(pgs->memory, (byte *)psrc_decode, "image_render_color_icc");
+                if (code < 0)
+                    return code;
             } else {
                 /* CM only. No decode */
-                (penum->icc_link->procs.map_buffer)(dev, penum->icc_link, 
+                code = (penum->icc_link->procs.map_buffer)(dev, penum->icc_link,
                                                     &input_buff_desc,
-                                                    &output_buff_desc, 
-                                                    (void*) psrc, 
+                                                    &output_buff_desc,
+                                                    (void*) psrc,
                                                     (void*) psrc_cm);
+                if (code < 0)
+                    return code;
             }
         }
     }
@@ -711,11 +718,15 @@ image_render_icc16(gx_image_enum * penum, const byte * buffer, int data_x,
                 break;
             if (memcmp(run, psrc_cm, spp_cm * 2))
                 break;
-            if (posture == image_skewed)
+            if (posture == image_skewed) {
+                if (first) /* We always need to map the first pixel on a line */
+                    break;
                 /* The color doesn't need remapping, but we can't handle a run */
                 goto skewed_but_same;
+            }
             run += spp_cm;
         }
+        first = 0;
         /* So we have a run of pixels from psrc_cm to run that are all the same. */
         /* This needs to be sped up */
         for ( k = 0; k < spp_cm; k++ ) {
