@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -22,6 +22,9 @@
 #include "gxcpath.h"
 #include "gximage.h"
 #include "gsicc_cache.h"
+#ifdef WITH_CAL
+#include "cal.h"
+#endif
 
 /* Forward declarations */
 static void update_strip(gx_image_enum *penum);
@@ -71,7 +74,7 @@ gx_image1_plane_data(gx_image_enum_common_t * info,
         int px;
 
         for (px = 0; px < num_planes; ++px)
-            offsets[px] = planes[px].raster * penum->used.y;
+            offsets[px] = (size_t)planes[px].raster * penum->used.y;
         penum->used.y = 0;
     } else
         memset(offsets, 0, num_planes * sizeof(offsets[0]));
@@ -205,6 +208,7 @@ gx_image1_plane_data(gx_image_enum_common_t * info,
             dmprintf1(dev->memory, "[b]image1 y=%d\n", y);
         if (gs_debug_c('B')) {
             int i, n = width_spp;
+            byte *buftemp = (buffer == NULL) ? penum->buffer : (byte *)buffer;
 
             if (penum->bps > 8)
                 n *= 2;
@@ -212,7 +216,7 @@ gx_image1_plane_data(gx_image_enum_common_t * info,
                 n = (n + 7) / 8;
             dmlputs(dev->memory, "[B]row:");
             for (i = 0; i < n; i++)
-                dmprintf1(dev->memory, " %02x", buffer[i]);
+                dmprintf1(dev->memory, " %02x", buftemp[i]);
             dmputs(dev->memory, "\n");
         }
 #endif
@@ -517,8 +521,19 @@ gx_image1_end_image(gx_image_enum_common_t * info, bool draw_last)
     if (penum->clues != NULL) {
         gs_free_object(mem,penum->clues, "image clues");
     }
+
+    /* decrement this ref that was incremented in gx_image_enum_begin() */
+    rc_decrement_only(penum->pcs, "pcs");
+    penum->pcs = NULL;
+
     gs_free_object(mem, penum->line, "image line");
     gs_free_object(mem, penum->buffer, "image buffer");
+
+#ifdef WITH_CAL
+    if (penum->cal_ht != NULL) {
+        cal_halftone_fin(penum->cal_ht, mem->non_gc_memory);
+    }
+#endif
     gx_image_free_enum(&info);
     return 0;
 }
