@@ -71,45 +71,45 @@
  * The commented values are from the PC-PR1000E/4 User's Manual.
  */
 /* margins of A3 paper */
-#define L_MARGIN_A3 0.20
-#define B_MARGIN_A3 0.24
-#define R_MARGIN_A3 0.20
-#define T_MARGIN_A3 0.20
+#define L_MARGIN_A3 0.20f
+#define B_MARGIN_A3 0.24f
+#define R_MARGIN_A3 0.20f
+#define T_MARGIN_A3 0.20f
 /* margins of A4 paper */
-#define L_MARGIN_A4 0.31
-#define B_MARGIN_A4 0.20
-#define R_MARGIN_A4 0.16
-#define T_MARGIN_A4 0.20
+#define L_MARGIN_A4 0.31f
+#define B_MARGIN_A4 0.20f
+#define R_MARGIN_A4 0.16f
+#define T_MARGIN_A4 0.20f
 /* margins of A5 paper */
-#define L_MARGIN_A5 0.31
-#define B_MARGIN_A5 0.16
-#define R_MARGIN_A5 0.16
-#define T_MARGIN_A5 0.20
+#define L_MARGIN_A5 0.31f
+#define B_MARGIN_A5 0.16f
+#define R_MARGIN_A5 0.16f
+#define T_MARGIN_A5 0.20f
 /* margins of B4 paper */
-#define L_MARGIN_B4 0.31
-#define B_MARGIN_B4 0.24
-#define R_MARGIN_B4 0.31
-#define T_MARGIN_B4 0.20
+#define L_MARGIN_B4 0.31f
+#define B_MARGIN_B4 0.24f
+#define R_MARGIN_B4 0.31f
+#define T_MARGIN_B4 0.20f
 /* margins of B5 paper */
-#define L_MARGIN_B5 0.31
-#define B_MARGIN_B5 0.24
-#define R_MARGIN_B5 0.16
-#define T_MARGIN_B5 0.20
+#define L_MARGIN_B5 0.31f
+#define B_MARGIN_B5 0.24f
+#define R_MARGIN_B5 0.16f
+#define T_MARGIN_B5 0.20f
 /* margins of letter size paper */
-#define L_MARGIN_LETTER 0.31
-#define B_MARGIN_LETTER 0.24
-#define R_MARGIN_LETTER 0.20
-#define T_MARGIN_LETTER 0.20
+#define L_MARGIN_LETTER 0.31f
+#define B_MARGIN_LETTER 0.24f
+#define R_MARGIN_LETTER 0.20f
+#define T_MARGIN_LETTER 0.20f
 /* margins of postcard size paper */
-#define L_MARGIN_POSTCARD 0.31
-#define B_MARGIN_POSTCARD 0.12
-#define R_MARGIN_POSTCARD 0.24
-#define T_MARGIN_POSTCARD 0.20
+#define L_MARGIN_POSTCARD 0.31f
+#define B_MARGIN_POSTCARD 0.12f
+#define R_MARGIN_POSTCARD 0.24f
+#define T_MARGIN_POSTCARD 0.20f
 /* margins of env4 size paper */
-#define L_MARGIN_ENV4 0.20
-#define B_MARGIN_ENV4 0.20
-#define R_MARGIN_ENV4 0.20
-#define T_MARGIN_ENV4 0.20
+#define L_MARGIN_ENV4 0.20f
+#define B_MARGIN_ENV4 0.20f
+#define R_MARGIN_ENV4 0.20f
+#define T_MARGIN_ENV4 0.20f
 
 /* The device descriptors */
 static dev_proc_open_device(npdl_open);
@@ -118,12 +118,19 @@ static dev_proc_print_page_copies(npdl_print_page_copies);
 static dev_proc_put_params(npdl_put_params);
 static dev_proc_image_out(npdl_image_out);
 
-static gx_device_procs npdl_prn_procs =
-prn_params_procs(npdl_open, gdev_prn_output_page, npdl_close,
-                 lprn_get_params, npdl_put_params);
+static void
+npdl_initialize_device_procs(gx_device *dev)
+{
+    gdev_prn_initialize_device_procs_mono(dev);
+
+    set_dev_proc(dev, open_device, npdl_open);
+    set_dev_proc(dev, close_device, npdl_close);
+    set_dev_proc(dev, get_params, lprn_get_params);
+    set_dev_proc(dev, put_params, npdl_put_params);
+}
 
 gx_device_lprn far_data gs_npdl_device =
-lprn_duplex_device(gx_device_lprn, npdl_prn_procs, "npdl",
+lprn_duplex_device(gx_device_lprn, npdl_initialize_device_procs, "npdl",
             X_DPI, Y_DPI,	/* default resolution */
             0.0, 0.0, 0.0, 0.0,	/* margins */
             1, npdl_print_page_copies, npdl_image_out);
@@ -527,8 +534,8 @@ npdl_set_page_layout(gx_device * dev)
 static int
 npdl_open(gx_device * dev)
 {
-    int xdpi = dev->x_pixels_per_inch;
-    int ydpi = dev->y_pixels_per_inch;
+    int xdpi = (int)dev->x_pixels_per_inch;
+    int ydpi = (int)dev->y_pixels_per_inch;
 
     /* Print Resolution Check */
     if (xdpi != ydpi)
@@ -548,7 +555,7 @@ npdl_close(gx_device *pdev)
     gx_device_printer *const ppdev = (gx_device_printer *) pdev;
     int code = gdev_prn_open_printer(pdev, 1);
     if (code >= 0)
-        fputs("\033c1", ppdev->file);
+        gp_fputs("\033c1", ppdev->file);
 
     return gdev_prn_close(pdev);
 }
@@ -571,7 +578,7 @@ npdl_put_params(gx_device * pdev, gs_param_list * plist)
 /* Send the page to the printer.  For speed, compress each scan line,
    since computer-to-printer communication time is often a bottleneck. */
 static int
-npdl_print_page_copies(gx_device_printer * pdev, FILE * prn_stream, int num_copies)
+npdl_print_page_copies(gx_device_printer * pdev, gp_file * prn_stream, int num_copies)
 {
     gx_device_lprn *const lprn = (gx_device_lprn *) pdev;
     int line_size = gdev_prn_raster(pdev);
@@ -587,8 +594,8 @@ npdl_print_page_copies(gx_device_printer * pdev, FILE * prn_stream, int num_copi
     if (pdev->PageCount == 0) {
 
       /* Initialize printer */
-      fputs("\033c1", prn_stream);               /* Software Reset */
-      fputs("\034d240.", prn_stream);            /* Page Printer Mode */
+      gp_fputs("\033c1", prn_stream);               /* Software Reset */
+      gp_fputs("\034d240.", prn_stream);            /* Page Printer Mode */
 
         /* Check paper size */
         switch (npdl_get_paper_size((gx_device *) pdev)) {
@@ -622,58 +629,58 @@ npdl_print_page_copies(gx_device_printer * pdev, FILE * prn_stream, int num_copi
         }
 
         if (lprn->ManualFeed) {
-        fprintf(prn_stream, "\034f%cM0.",
-                (pdev->MediaSize[0] > pdev->MediaSize[1]) ? 'L' : 'P');
+        gp_fprintf(prn_stream, "\034f%cM0.",
+                   (pdev->MediaSize[0] > pdev->MediaSize[1]) ? 'L' : 'P');
         /* Page Orientation  P: Portrait, L: Landscape */
         } else {
-        fprintf(prn_stream, "\034f%c%s.",
-                (pdev->MediaSize[0] > pdev->MediaSize[1]) ? 'L' : 'P',
+        gp_fprintf(prn_stream, "\034f%c%s.",
+                   (pdev->MediaSize[0] > pdev->MediaSize[1]) ? 'L' : 'P',
         /* Page Orientation  P: Portrait, L: Landscape */
-                paper_command);	/* Paper Size */
+                   paper_command);	/* Paper Size */
         }
 
-        fprintf(prn_stream, "\034<1/%d,i.", x_dpi);	/* Image Resolution */
+        gp_fprintf(prn_stream, "\034<1/%d,i.", x_dpi);	/* Image Resolution */
 
         /* Duplex Setting */
         if (pdev->Duplex_set > 0) {
             if (pdev->Duplex) {
                 if (lprn->Tumble == 0)
-                  fprintf(prn_stream, "\034'B,,1,0.");
+                  gp_fprintf(prn_stream, "\034'B,,1,0.");
                 else
-                  fprintf(prn_stream, "\034'B,,2,0.");
+                  gp_fprintf(prn_stream, "\034'B,,2,0.");
             } else
-              fprintf(prn_stream, "\034'S,,,0.");
+              gp_fprintf(prn_stream, "\034'S,,,0.");
         }
     }
 
     if (num_copies > 99)
        num_copies = 99;
-    fprintf(prn_stream, "\034x%d.", num_copies);
+    gp_fprintf(prn_stream, "\034x%d.", num_copies);
 
     lprn->initialized = false;
 
     if (lprn->NegativePrint) {
-        fprintf(prn_stream, "\034e0,0.");	/* move to (0, 0) */
-        fprintf(prn_stream, "\034Y");	/* goto figure mode */
-        fprintf(prn_stream, "SU1,%d,0;", (int)pdev->x_pixels_per_inch);
+        gp_fprintf(prn_stream, "\034e0,0.");	/* move to (0, 0) */
+        gp_fprintf(prn_stream, "\034Y");	/* goto figure mode */
+        gp_fprintf(prn_stream, "SU1,%d,0;", (int)pdev->x_pixels_per_inch);
         /* Setting Printer Unit */
-        fprintf(prn_stream, "SG0,0;");	/* select black color */
-        fprintf(prn_stream, "NP;");	/* begin path */
-        fprintf(prn_stream, "PA%d,0,%d,%d,0,%d;",
-                pdev->width, pdev->width, pdev->height, pdev->height);
+        gp_fprintf(prn_stream, "SG0,0;");	/* select black color */
+        gp_fprintf(prn_stream, "NP;");	/* begin path */
+        gp_fprintf(prn_stream, "PA%d,0,%d,%d,0,%d;",
+                   pdev->width, pdev->width, pdev->height, pdev->height);
         /* draw rectangle */
-        fprintf(prn_stream, "CP");	/* close path */
-        fprintf(prn_stream, "EP;");	/* end path */
-        fprintf(prn_stream, "FL0;");	/* fill path */
-        fprintf(prn_stream, "\034Z");	/* end of figure mode */
-        fprintf(prn_stream, "\034\"R.");	/* `R'eplace Mode */
+        gp_fprintf(prn_stream, "CP");	/* close path */
+        gp_fprintf(prn_stream, "EP;");	/* end path */
+        gp_fprintf(prn_stream, "FL0;");	/* fill path */
+        gp_fprintf(prn_stream, "\034Z");	/* end of figure mode */
+        gp_fprintf(prn_stream, "\034\"R.");	/* `R'eplace Mode */
     }
     code = lprn_print_image(pdev, prn_stream);
     if (code < 0)
         return code;
 
     /* Form Feed */
-    fputs("\014", prn_stream);
+    gp_fputs("\014", prn_stream);
 
     gs_free(pdev->memory->non_gc_memory, lprn->CompBuf, line_size * maxY, sizeof(byte), "npdl_print_page_copies(CompBuf)");
     return 0;
@@ -681,13 +688,13 @@ npdl_print_page_copies(gx_device_printer * pdev, FILE * prn_stream, int num_copi
 
 /* Output data */
 static void
-npdl_image_out(gx_device_printer * pdev, FILE * prn_stream, int x, int y, int width, int height)
+npdl_image_out(gx_device_printer * pdev, gp_file * prn_stream, int x, int y, int width, int height)
 {
     gx_device_lprn *const lprn = (gx_device_lprn *) pdev;
     int num_bytes;
     int x_dpi = (int)(pdev->x_pixels_per_inch);
 
-    fprintf(prn_stream, "\034e%d,%d.", x, y);
+    gp_fprintf(prn_stream, "\034e%d,%d.", x, y);
     /* Data compression */
     num_bytes = mh_compression(lprn->TmpBuf, lprn->CompBuf, width / 8, height);
 
@@ -695,16 +702,16 @@ npdl_image_out(gx_device_printer * pdev, FILE * prn_stream, int x, int y, int wi
      * If the compression ratio >= 100%, send uncompressed data
      */
     if (num_bytes == 0) {
-        fprintf(prn_stream, "\034i%d,%d,0,1/1,1/1,%d,%d.", width,
-                height, width * height / 8, x_dpi);
-        fwrite(lprn->TmpBuf, 1, width * height / 8, prn_stream);
+        gp_fprintf(prn_stream, "\034i%d,%d,0,1/1,1/1,%d,%d.", width,
+                   height, width * height / 8, x_dpi);
+        gp_fwrite(lprn->TmpBuf, 1, width * height / 8, prn_stream);
     }
     /*
      * If the compression ratio < 100%, send compressed data
      */
     else {
-        fprintf(prn_stream, "\034i%d,%d,1,1/1,1/1,%d,%d.", width,
-                height, num_bytes, x_dpi);
-        fwrite(lprn->CompBuf, 1, num_bytes, prn_stream);
+        gp_fprintf(prn_stream, "\034i%d,%d,1,1/1,1/1,%d,%d.", width,
+                   height, num_bytes, x_dpi);
+        gp_fwrite(lprn->CompBuf, 1, num_bytes, prn_stream);
     }
 }
