@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -202,8 +202,13 @@ pdf_xmp_time(char *buf, int buf_length)
     time_t t;
     char buf1[4+1+2+1+2+1]; /* yyyy-mm-dd\0 */
 
+#ifdef CLUSTER
+    memset(&t, 0, sizeof(t));
+    memset(&tms, 0, sizeof(tms));
+#else
     time(&t);
     tms = *localtime(&t);
+#endif
     gs_sprintf(buf1,
             "%04d-%02d-%02d",
             tms.tm_year + 1900, tms.tm_mon + 1, tms.tm_mday);
@@ -297,7 +302,7 @@ pdf_get_docinfo_item(gx_device_pdf *pdev, const char *key, char *buf, int buf_le
 }
 
 static inline byte
-decode_escape(const byte *data, int data_length, int *index)
+decode_escape(const byte *data, int data_length, size_t *index)
 {
     byte c;
 
@@ -351,14 +356,14 @@ decode_escape(const byte *data, int data_length, int *index)
  */
 static const char firstByteMark[7] = { 0x00, 0x00, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC };
 
-static int gs_ConvertUTF16(unsigned char *UTF16, int UTF16Len, unsigned char **UTF8Start, int UTF8Len)
+static int gs_ConvertUTF16(unsigned char *UTF16, size_t UTF16Len, unsigned char **UTF8Start, int UTF8Len)
 {
-    int i, bytes = 0;
+    size_t i, bytes = 0;
     unsigned short U16;
     unsigned char *UTF8 = *UTF8Start;
     unsigned char *UTF8End = UTF8 + UTF8Len;
 
-    if (fabs(UTF16Len % sizeof(short)) != 0)
+    if (UTF16Len % sizeof(short) != 0)
         return gs_note_error(gs_error_rangecheck);
 
     for (i=0;i<UTF16Len / sizeof(short);i++)
@@ -414,7 +419,7 @@ static int
 pdf_xmp_write_translated(gx_device_pdf *pdev, stream *s, const byte *data, int data_length,
                          void(*write)(stream *s, const byte *data, int data_length))
 {
-    int i, j=0;
+    size_t i, j=0;
     unsigned char *buf0;
 
     if (data_length == 0)
@@ -424,7 +429,7 @@ pdf_xmp_write_translated(gx_device_pdf *pdev, stream *s, const byte *data, int d
                     "pdf_xmp_write_translated");
     if (buf0 == NULL)
         return_error(gs_error_VMerror);
-    for (i = 0; i < data_length; i++) {
+    for (i = 0; i < (size_t)data_length; i++) {
         byte c = data[i];
 
         if (c == '\\')
@@ -487,6 +492,7 @@ pdf_xmp_write_translated(gx_device_pdf *pdev, stream *s, const byte *data, int d
         if (code < 0)
             return code;
         write(s, (const byte *)buf1, buf1b - buf1);
+        gs_free_object(pdev->memory, buf1, "pdf_xmp_write_translated");
     }
     gs_free_object(pdev->memory, buf0, "pdf_xmp_write_translated");
     return 0;
@@ -646,8 +652,7 @@ pdf_write_document_metadata(gx_device_pdf *pdev, const byte digest[6])
         {
 
             pdf_xml_tag_open_beg(s, "rdf:Description");
-            pdf_xml_attribute_name(s, "rdf:about");
-            pdf_xml_attribute_value(s, instance_uuid);
+            pdf_xml_copy(s, " rdf:about=\"\"");
             pdf_xml_attribute_name(s, "xmlns:pdf");
             pdf_xml_attribute_value(s, "http://ns.adobe.com/pdf/1.3/");
 
@@ -684,8 +689,7 @@ pdf_write_document_metadata(gx_device_pdf *pdev, const byte digest[6])
             }
 
             pdf_xml_tag_open_beg(s, "rdf:Description");
-            pdf_xml_attribute_name(s, "rdf:about");
-            pdf_xml_attribute_value(s, instance_uuid);
+            pdf_xml_copy(s, " rdf:about=\"\"");
             pdf_xml_attribute_name(s, "xmlns:xmp");
             pdf_xml_attribute_value(s, "http://ns.adobe.com/xap/1.0/");
             pdf_xml_tag_end(s);
@@ -718,8 +722,7 @@ pdf_write_document_metadata(gx_device_pdf *pdev, const byte digest[6])
             pdf_xml_newline(s);
 
             pdf_xml_tag_open_beg(s, "rdf:Description");
-            pdf_xml_attribute_name(s, "rdf:about");
-            pdf_xml_attribute_value(s, instance_uuid);
+            pdf_xml_copy(s, " rdf:about=\"\"");
             pdf_xml_attribute_name(s, "xmlns:xapMM");
             pdf_xml_attribute_value(s, "http://ns.adobe.com/xap/1.0/mm/");
             pdf_xml_attribute_name(s, "xapMM:DocumentID");
@@ -728,8 +731,7 @@ pdf_write_document_metadata(gx_device_pdf *pdev, const byte digest[6])
             pdf_xml_newline(s);
 
             pdf_xml_tag_open_beg(s, "rdf:Description");
-            pdf_xml_attribute_name(s, "rdf:about");
-            pdf_xml_attribute_value(s, instance_uuid);
+            pdf_xml_copy(s, " rdf:about=\"\"");
             pdf_xml_attribute_name(s, "xmlns:dc");
             pdf_xml_attribute_value(s, "http://purl.org/dc/elements/1.1/");
             pdf_xml_attribute_name(s, "dc:format");
@@ -803,8 +805,7 @@ pdf_write_document_metadata(gx_device_pdf *pdev, const byte digest[6])
             pdf_xml_newline(s);
             if (pdev->PDFA != 0) {
                 pdf_xml_tag_open_beg(s, "rdf:Description");
-                pdf_xml_attribute_name(s, "rdf:about");
-                pdf_xml_attribute_value(s, instance_uuid);
+                pdf_xml_copy(s, " rdf:about=\"\"");
                 pdf_xml_attribute_name(s, "xmlns:pdfaid");
                 pdf_xml_attribute_value(s, "http://www.aiim.org/pdfa/ns/id/");
                 pdf_xml_attribute_name(s, "pdfaid:part");
@@ -840,8 +841,6 @@ pdf_write_document_metadata(gx_device_pdf *pdev, const byte digest[6])
 int
 pdf_document_metadata(gx_device_pdf *pdev)
 {
-    int j;
-
     if (pdev->CompatibilityLevel < 1.4)
         return 0;
     if (cos_dict_find_c_key(pdev->Catalog, "/Metadata"))
@@ -889,33 +888,6 @@ pdf_document_metadata(gx_device_pdf *pdev)
         if (code < 0)
             return code;
 
-        /* By storing the Metadata in the Catalog dictionary we have referenced it from the
-         * 'global named objects', and the object will be freed when we free those objects.
-         * We *don't* want it referenced by any resource because if it is then we will free
-         * it before we free the global named resources. So remove the Metadata cos_stream
-         * object from the resourceOther resource type chains.
-         */
-        for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
-            pdf_resource_t *pres1, *head = pdev->resources[resourceOther].chains[j];
-
-            pres1 = head;
-
-            if (pres1 == pres) {
-                pdev->resources[resourceOther].chains[j] = pres1->next;
-                break;
-            } else {
-                while (pres1->next) {
-                    if (pres1->next == pres) {
-                        pres1->next = pres->next;
-                        break;
-                    }
-                    if (pres1->next == pres->next)
-                        break;
-
-                    pres1 = pres1->next;
-                }
-            }
-        }
     }
     return 0;
 }
