@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -49,7 +49,7 @@ Boolean   gDone;
 ControlActionUPP gActionFunctionScrollUPP;
 
 const char start_string[] = "systemdict /start get exec\n";
-void *instance;
+void *instance = NULL;
 
 const unsigned int display_format = DISPLAY_COLORS_RGB | DISPLAY_UNUSED_FIRST |
                                     DISPLAY_DEPTH_8 | DISPLAY_BIGENDIAN |
