@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -19,8 +19,10 @@
 #ifndef ivmem2_INCLUDED
 #  define ivmem2_INCLUDED
 
+#include "iref.h"
+
 /* Exported by zvmem2.c for zusparam.c */
 int set_vm_reclaim(i_ctx_t *, long);
-int set_vm_threshold(i_ctx_t *, long);
+int set_vm_threshold(i_ctx_t *, int64_t);
 
 #endif /* ivmem2_INCLUDED */
