@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2021 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -26,6 +26,7 @@
 #include "gxiparam.h"		/* for image enumerator */
 #include "gxcspace.h"
 #include "idict.h"
+#include "idstack.h"
 #include "idparam.h"
 #include "ifunc.h"
 #include "igstate.h"
@@ -41,20 +42,6 @@
 
 /* ------ Utilities ------ */
 
-static int
-set_float_value(i_ctx_t *i_ctx_p, int (*set_value)(gs_gstate *, double))
-{
-    os_ptr op = osp;
-    double value;
-    int code;
-
-    if (real_param(op, &value) < 0)
-        return_op_typecheck(op);
-    if ((code = set_value(igs, value)) < 0)
-        return code;
-    pop(1);
-    return 0;
-}
 
 static int
 current_float_value(i_ctx_t *i_ctx_p,
@@ -123,34 +110,6 @@ zcurrentblendmode(i_ctx_t *i_ctx_p)
     return 0;
 }
 
-/* <0..1> .setopacityalpha - */
-static int
-zsetopacityalpha(i_ctx_t *i_ctx_p)
-{
-    return set_float_value(i_ctx_p, gs_setopacityalpha);
-}
-
-/* - .currentopacityalpha <0..1> */
-static int
-zcurrentopacityalpha(i_ctx_t *i_ctx_p)
-{
-    return current_float_value(i_ctx_p, gs_currentopacityalpha);
-}
-
-/* <0..1> .setshapealpha - */
-static int
-zsetshapealpha(i_ctx_t *i_ctx_p)
-{
-    return set_float_value(i_ctx_p, gs_setshapealpha);
-}
-
-/* - .currentshapealpha <0..1> */
-static int
-zcurrentshapealpha(i_ctx_t *i_ctx_p)
-{
-    return current_float_value(i_ctx_p, gs_currentshapealpha);
-}
-
 /* <bool> .settextknockout - */
 static int
 zsettextknockout(i_ctx_t *i_ctx_p)
@@ -205,9 +164,7 @@ mask_op(i_ctx_t *i_ctx_p,
 
 }
 
-/* <paramdict> <llx> <lly> <urx> <ury> .begintransparencygroup - */
-static int
-zbegintransparencygroup(i_ctx_t *i_ctx_p)
+static int common_transparency_group(i_ctx_t *i_ctx_p, pdf14_compositor_operations group_type)
 {
     os_ptr op = osp;
     os_ptr dop = op - 4;
@@ -218,7 +175,7 @@ zbegintransparencygroup(i_ctx_t *i_ctx_p)
 
     check_type(*dop, t_dictionary);
     check_dict_read(*dop);
-    gs_trans_group_params_init(&params);
+    gs_trans_group_params_init(&params, 1.0);
     if ((code = dict_bool_param(dop, "Isolated", false, &params.Isolated)) < 0 ||
         (code = dict_bool_param(dop, "Knockout", false, &params.Knockout)) < 0 ||
         (code = dict_bool_param(dop, ".image_with_SMask", false, &params.image_with_SMask)) < 0
@@ -247,13 +204,36 @@ zbegintransparencygroup(i_ctx_t *i_ctx_p)
                 params.ColorSpace = NULL;
         }
     }
-    code = gs_begin_transparency_group(igs, &params, &bbox);
+
+    if (gs_getalphaisshape(igs)) {
+        params.group_shape = gs_getfillconstantalpha(igs);
+        params.group_opacity = 1.0;
+    } else {
+        params.group_opacity = gs_getfillconstantalpha(igs);
+        params.group_shape = 1.0;
+    }
+
+    code = gs_begin_transparency_group(igs, &params, &bbox, group_type);
     if (code < 0)
         return code;
     pop(5);
     return code;
 }
 
+/* <paramdict> <llx> <lly> <urx> <ury> .beginpagegroup - */
+static int
+zbegintransparencypagegroup(i_ctx_t *i_ctx_p)
+{
+    return common_transparency_group(i_ctx_p, PDF14_BEGIN_TRANS_PAGE_GROUP);
+}
+
+/* <paramdict> <llx> <lly> <urx> <ury> .begintransparencygroup - */
+static int
+zbegintransparencygroup(i_ctx_t *i_ctx_p)
+{
+    return common_transparency_group(i_ctx_p, PDF14_BEGIN_TRANS_GROUP);
+}
+
 /* - .endtransparencygroup - */
 static int
 zendtransparencygroup(i_ctx_t *i_ctx_p)
@@ -420,7 +400,7 @@ zimage3x(i_ctx_t *i_ctx_p)
     check_type(*pDataDict, t_dictionary);
     if ((code = pixel_image_params(i_ctx_p, pDataDict,
                    (gs_pixel_image_t *)&image, &ip_data,
-                   16, false, gs_currentcolorspace(igs))) < 0 ||
+                   16, gs_currentcolorspace(igs))) < 0 ||
         (code = dict_int_param(pDataDict, "ImageType", 1, 1, 0, &ignored)) < 0
         )
         return code;
@@ -458,7 +438,7 @@ image_params *pip_data, const char *dict_name,
         return gs_note_error(gs_error_typecheck);
 
     if ((mcode = code = data_image_params(mem, pMaskDict, &pixm->MaskDict,
-                                          &ip_mask, false, 1, 16, false, false)) < 0 ||
+                                          &ip_mask, false, 1, 16, false)) < 0 ||
         (code = dict_int_param(pMaskDict, "ImageType", 1, 1, 0, &ignored)) < 0 ||
         (code = dict_int_param(pMaskDict, "InterleaveType", 1, 3, -1,
                                &pixm->InterleaveType)) < 0 ||
@@ -492,15 +472,27 @@ static int
 zpushpdf14devicefilter(i_ctx_t *i_ctx_p)
 {
     int code;
+    int depth;
+    int spot_color_count = -1;		/* default is 'unknown' spot color count */
     os_ptr op = osp;
     gx_device *cdev = gs_currentdevice_inline(igs);
+    dict_stack_t *dstack = &(i_ctx_p->dict_stack);
+    ref_stack_t *rdstack = &dstack->stack;
+    const ref *puserdict = ref_stack_index(rdstack, ref_stack_count(rdstack) - 1 -
+                            dstack->userdict_index);
 
     check_type(*op, t_integer);
+    depth = (int)op->value.intval;
+
+    if (dev_proc(cdev, dev_spec_op)(cdev, gxdso_is_pdf14_device, NULL, 0) > 0)
+        return 0;		/* ignore push_device if already is pdf14 device */
+
     /* Bug 698087: In case some program uses our .pushpdf14devicefilter  make	*/
     /*             sure that the device knows that we are using the pdf14	*/
     /*             transparency. Note this will close and re-open the device	*/
     /*             and erase the page. This should not occur with PDF files.	*/
-    if (cdev->page_uses_transparency == 0) {
+    /* We don't do this if this is a push for the overprint_sim mode		*/
+    if (depth >= 0 && cdev->page_uses_transparency == 0) {
         gs_c_param_list list;
         bool bool_true = 1;
 
@@ -516,15 +508,18 @@ zpushpdf14devicefilter(i_ctx_t *i_ctx_p)
         if (cdev->is_open) {
             if ((code = gs_closedevice((gx_device *)cdev)) < 0)
                 return code;
-            if (dev_proc(cdev, dev_spec_op)(cdev, gxdso_is_pdf14_device, NULL, 0) > 0)
-                pdf14_disable_device(cdev);	/* should already be disabled  (bug 698306) */
         }
         if ((code = gs_opendevice((gx_device *)cdev)) < 0)
             return code;
         if ((code = gs_erasepage(igs)) < 0)
             return code;
     }
-    code = gs_push_pdf14trans_device(igs, false);
+    /* Get the PageSpotColors value from the userdict, if it is defined */
+    code = dict_int_param(puserdict, "PageSpotColors", -1, max_int, -1, &spot_color_count);
+    if (code < 0)
+        return code;
+    /* and finally actually push the compositor device */
+    code = gs_push_pdf14trans_device(igs, false, true, depth, spot_color_count);
     if (code < 0)
         return code;
     pop(1);
@@ -662,10 +657,6 @@ zcurrentSMask(i_ctx_t *i_ctx_p)
 const op_def ztrans1_op_defs[] = {
     {"1.setblendmode", zsetblendmode},
     {"0.currentblendmode", zcurrentblendmode},
-    {"1.setopacityalpha", zsetopacityalpha},
-    {"0.currentopacityalpha", zcurrentopacityalpha},
-    {"1.setshapealpha", zsetshapealpha},
-    {"0.currentshapealpha", zcurrentshapealpha},
     {"1.settextknockout", zsettextknockout},
     {"0.currenttextknockout", zcurrenttextknockout},
     {"0.pushextendedgstate", zpushextendedgstate},
@@ -674,6 +665,7 @@ const op_def ztrans1_op_defs[] = {
 };
 const op_def ztrans2_op_defs[] = {
     {"5.begintransparencygroup", zbegintransparencygroup},
+    {"5.begintransparencypagegroup", zbegintransparencypagegroup},
     {"0.endtransparencygroup", zendtransparencygroup},
     { "0.endtransparencytextgroup", zendtransparencytextgroup },
     { "0.begintransparencytextgroup", zbegintransparencytextgroup },
